/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.awt.Component;
import java.util.List;
import java.util.NoSuchElementException;
import javax.swing.JComponent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.fleet.machine.MachineDataService.Model;
import nl.cloudfarming.client.fleet.machine.MachineDataService.StepInfo;
import org.netbeans.api.project.Project;
import org.openide.WizardDescriptor;
import org.openide.util.NbBundle;

/**
 * The default iterator for machine creation wizards. It provides functionality
 * for stepping through the wizard. Implement a new MachineWizardFactory to
 * implement which panels this iterator should contain initially. When the user
 * selects model in the wizard, additional steps for that model are added to the
 * wizard.
 *
 * @author Wytse Visser
 */
@NbBundle.Messages({"MachineWizardIterator name={0} from {1}"})
public class MachineWizardIterator implements WizardDescriptor.Iterator<WizardDescriptor> {

    private int index;
    private List<WizardDescriptor.Panel<WizardDescriptor>> panels;
    private MachineWizardFactory factory;
    public static final String PROP_MODEL = "model";
    public static final String PROP_MACHINE = "machine";
    private int firstDynamicStep;

    public MachineWizardIterator(MachineWizardFactory factory) {
        this.factory = factory;
    }

    private List<WizardDescriptor.Panel<WizardDescriptor>> getPanels() {
        if (panels == null) {
            panels = factory.createInitialPanels();
            firstDynamicStep = panels.size();

            panels.add(new MachineUnknownStepsLeftPanel());
            updateSteps();
        }
        return panels;
    }

    @Override
    public WizardDescriptor.Panel<WizardDescriptor> current() {
        return getPanels().get(index);
    }

    @Override
    public String name() {
        return Bundle.MachineWizardIterator_name(index + 1, getPanels().size());
    }

    @Override
    public boolean hasNext() {
        return index < getPanels().size() - 1;
    }

    @Override
    public boolean hasPrevious() {
        return index > 0;
    }

    @Override
    public void nextPanel() {
        if (!hasPrevious()) {
            addDynamicPanels();
            updateSteps();
        }
        if (!hasNext()) {
            throw new NoSuchElementException();
        }
        index++;
    }

    @Override
    public void previousPanel() {
        if (!hasPrevious()) {
            throw new NoSuchElementException();
        }
        index--;
    }

    @Override
    public void addChangeListener(ChangeListener l) {
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
    }

    private void updateSteps() {
        String[] steps = new String[panels.size()];
        for (int i = 0; i < panels.size(); i++) {
            Component c = panels.get(i).getComponent();
            steps[i] = c.getName();
            if (c instanceof JComponent) {
                JComponent jc = (JComponent) c;
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_SELECTED_INDEX, i);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DATA, steps);
                jc.putClientProperty(WizardDescriptor.PROP_AUTO_WIZARD_STYLE, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_DISPLAYED, true);
                jc.putClientProperty(WizardDescriptor.PROP_CONTENT_NUMBERED, true);
            }
        }
    }

    /**
     * Checks if a model is selected and adds the dynamic panels for that model
     * to this iterator.
     */
    protected void addDynamicPanels() {
        Model model = getSelectedModel();
        if (model != null) {
            for (int i = panels.size() - 1; i >= firstDynamicStep; i--) {
                panels.remove(i);
            }

            List<StepInfo> steps = ((Model) model).getAdditionalSteps();
            int stepIndex = firstDynamicStep;
            for (StepInfo step : steps) {
                panels.add(stepIndex, new MachineDynamicWizardPanel(step));
                stepIndex++;
            }
            panels.add(factory.createOverviewPanel());
        }
    }

    /**
     * @return The selected machine model
     */
    private Model getSelectedModel() {
        WizardDescriptor wiz = new WizardDescriptor(this);
        current().storeSettings(wiz);
        return (Model) wiz.getProperty(PROP_MODEL);
    }

    /**
     * @return The project this WizardIterator belongs to
     */
    protected Project getProject() {
        return factory.getProject();
    }
}
